from PIL import Image, ImageDraw
import os
import glob
import random
import sys
import numpy as np

folder_name, num_samples = sys.argv[1:] 

if num_samples.isdigit():
    num_samples = int(num_samples)
    if num_samples <= 0:
        raise ValueError("Give num_samples more than 0")
else:
    raise ValueError("Give num_samples as an integer")

folder_name += '//'
if not os.path.exists(folder_name):
    print("'{}' doesn't exist, so a folder is created with that name at '{}'".format(folder_name, os.path.abspath(folder_name)))
    os.mkdir(folder_name)

symbol_images = glob.glob('symbols/*')

print('Data generation of {} images has started'.format(num_samples))

for idx in range(num_samples):
    image_name = random.choice(symbol_images)
    symbol_letter = os.path.basename(image_name)[0].upper()

    #tweakable parameters 
    #------------------------------------------------------------------------------
    line_width = random.randint(1, 6)
    rotate_angle = random.randrange(-40, 40)
    
    x_push = random.randint(0, 50)
    y_push = random.randint(0, 50)
    size = random.uniform(1, 2)
    num_strayMarks = random.choice([0, 1, 2])
    #------------------------------------------------------------------------------

    arr = np.asarray(Image.open(image_name).convert('L').rotate(rotate_angle, fillcolor = 'white', expand = True))
    points = np.transpose(np.asarray(arr==0).nonzero())
    points = tuple(map(lambda x: tuple([x[0]*size + x_push + line_width, x[1]*size + y_push + line_width]), points))

    multiplier = int(-(max(max(map(lambda x: x[0], points)), max(map(lambda x: x[1], points)))//-100))+1
    img_size = 100*multiplier

    im = Image.new("RGB", (img_size, img_size), 'WHITE')
    dr = ImageDraw.Draw(im)
    dr.line(points, fill="white", width=line_width)
    for point in points:
        dr.ellipse((point[1] - line_width, point[0] - line_width, point[1]  + line_width, point[0] + line_width), fill="black")
        
    #stray marks
    im = im.resize((32, 32))
    dr = ImageDraw.Draw(im)
    for _ in range(num_strayMarks):
        strayX, strayY = random.randint(0, 24), random.randint(0, 24)
        dr.point((strayX, strayY), fill="black")
        
    im.save(folder_name+"{}_{}.png".format(idx+1, symbol_letter))

    
print('Finished generating the images')
print('Images can be found in the path -> ', os.path.abspath(folder_name))